﻿#pragma once

#include <cigui/config.h>
#include <filesystem>
#include <LLGL/LLGL.h>

#include <cigui/utils/List.hpp>
#include <cigui/utils/Vectors.hpp>

namespace cig
{
	class LocalFilesystem
	{
		std::filesystem::path executable_path;
		std::filesystem::path working_directory;
	public:
		explicit LocalFilesystem(const char* arg0)
		{
			executable_path = std::filesystem::current_path() / arg0;
			working_directory = std::filesystem::current_path();
		}
	};

	struct alignas(4) Vertex
	{
		float position[2]; // 2D vector for X and Y coordinates
		uint8_t color[4]; // 4D vector for red, green, blue, and alpha components
	};

	class App
	{
	protected:
		LLGL::RenderSystemPtr m_RenderSystem;
		std::pair<LLGL::Shader*, LLGL::Shader*> m_ShaderPack;

		LLGL::SwapChain* m_SwapChain		= nullptr;
		LLGL::PipelineState* m_Pipeline		= nullptr;
		LLGL::Buffer* m_VertexBuffer		= nullptr;
		LLGL::CommandBuffer* m_CmdBuffer	= nullptr;
		LLGL::Window* m_Window				= nullptr;

		class WindowListener final : public LLGL::Window::EventListener
		{
		public:
			explicit WindowListener(App& app);

			void OnResize(LLGL::Window& sender, const LLGL::Extent2D& clientAreaSize) override;

			void OnQuit(LLGL::Window& sender, bool& veto) override;

			void OnLostFocus(LLGL::Window& sender) override;

			void OnGetFocus(LLGL::Window& sender) override;
		private:
			App& m_App;
		};
		std::shared_ptr<WindowListener> m_WindowListener;

		[[nodiscard]] bool ShaderLanguageIsSupported(LLGL::ShadingLanguage lang) const;

		List<Vertex> m_Vertices;

		vec2u m_WindowSize;
		void resized(vec2u size);

		void update(bool force = false);
		void render() const;

		[[nodiscard]] bool save() const;

	public:
		App();
		~App();

		void Initialize(int argc = 0, char** argv = nullptr);

		int Run();
	};
}
