﻿#pragma once

#include <cassert>
#include <algorithm>
#include <cstdlib>
#include <cigui/utils/MemoryDebug.hpp>


namespace cig::Memory
{
	template<typename T>
	T* alloc(const size_t count) {
		assert(count > 0 || "Count must be greater than 0");
		T* data = static_cast<T*>(calloc(count, sizeof(T)));
		MEMORY_ALLOCATOR_DEBUG(data, count);
		return data;
	}

	template<typename T>
	T* realloc(T* data, const size_t from, const size_t to)
	{
		assert((from > 0 && data) || "Data must exist");
		assert((to > 0 && to > from) ||"Destination must be greater than Source");
		T* newData = alloc<T>(to);
		std::copy(data, data + from, newData);
		dealloc<T>(data);
		MEMORY_REALLOCATOR_DEBUG(data, newData, to);
		return newData;
	}

	template<typename T>
	void dealloc(T* data)
	{
		assert(data || "Data must exist");
		MEMORY_DEALLOCATOR_DEBUG(data);
		free(data);
	}

	template<typename T>
	T* dupalloc(const T* data, const size_t count)
	{
		assert(data || "Data must exist");
		assert(count > 0 || "Count must be greater than 0");
		T* ndata = alloc<T>(count);
		std::copy(data, data + count, ndata);
		MEMORY_DUPALLOCATOR_DEBUG(data, count);
		return ndata;
	}
}