#pragma once

#include <cigui/config.h>

#include <stdint.h>


#define TYPEDEF_VECTOR(NAME, T, N, s) typedef NAME<T> vec##N##s;

#define TYPEDEF_VECTORS(NAME, N)              \
  typedef NAME<float> vec##N;                 \
  TYPEDEF_VECTOR(NAME, float, N, f)           \
  TYPEDEF_VECTOR(NAME, double, N, d)          \
  TYPEDEF_VECTOR(NAME, long double, N, ld)    \
  TYPEDEF_VECTOR(NAME, size_t, N, sz)         \
  TYPEDEF_VECTOR(NAME, int, N, i)             \
  TYPEDEF_VECTOR(NAME, unsigned int, N, u)    \
  TYPEDEF_VECTOR(NAME, short, N, s)           \
  TYPEDEF_VECTOR(NAME, unsigned short, N, us) \
  TYPEDEF_VECTOR(NAME, long, N, l)            \
  TYPEDEF_VECTOR(NAME, unsigned long, N, ul)  \
  TYPEDEF_VECTOR(NAME, long long, N, ll)      \
  TYPEDEF_VECTOR(NAME, unsigned long long, N, ull)


#if CIGUI_DLL
#define VECTOR_TEMPLATE_INST(N, T) CIGUI_TEMPLATE_INST Vector##N<T>; \

#define VECTOR_INSTANTIATION(N) \
	VECTOR_TEMPLATE_INST(N, float) \
	VECTOR_TEMPLATE_INST(N, double)\
	VECTOR_TEMPLATE_INST(N, long double) \
	VECTOR_TEMPLATE_INST(N, size_t) \
	VECTOR_TEMPLATE_INST(N, int) \
	VECTOR_TEMPLATE_INST(N, short) \
	VECTOR_TEMPLATE_INST(N, long) \
	VECTOR_TEMPLATE_INST(N, long long) \
	VECTOR_TEMPLATE_INST(N, unsigned int) \
	VECTOR_TEMPLATE_INST(N, unsigned short) \
	VECTOR_TEMPLATE_INST(N, unsigned long) \
	VECTOR_TEMPLATE_INST(N, unsigned long long)
#else
#define VECTOR_INSTANTIATION(N)
#endif


namespace cig
{
	template <typename T>
	union Vector2
	{
		struct
		{
			T x, y;
		} dims;

		struct
		{
			T width, height;
		} sizes;
	};

	TYPEDEF_VECTORS(Vector2, 2)

	template <typename T>
	union Vector3
	{
		struct
		{
			T x, y, z;
		} dims;

		struct
		{
			T r, g, b;
		} color;

		struct
		{
			T width, height, depth;
		} sizes;
	};

	TYPEDEF_VECTORS(Vector3, 3)

	template <typename T>
	union Vector4
	{
		struct
		{
			T x, y, z, w;
		} dims;

		struct
		{
			T r, g, b, a;
		} color;

		struct
		{
			T left, top, right, bottom;
		} dirs;
	};

	TYPEDEF_VECTORS(Vector4, 4)
}

#undef TYPEDEF_VECTOR
#undef TYPEDEF_VECTORS

#if CIGUI_DLL
#undef VECTOR_TEMPLATE_INST
#endif
#undef VECTOR_INSTANTIATION